from allauth.socialaccount import providers
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class SteemConnectAccount(ProviderAccount):
    def get_profile_url(self):
        return self.account.extra_data.get('html_url')

    def get_avatar_url(self):
        return self.account.extra_data.get('avatar_url')

    def to_str(self):
        dflt = super(SteemConnectAccount, self).to_str()
        return next(
            value
            for value in (
                self.account.extra_data.get('name', None),
                self.account.extra_data.get('login', None),
                dflt
            )
            if value is not None
        )


class SteemConnectProvider(OAuth2Provider):
    id = 'steemconnect'
    name = 'SteemConnect'
    account_class = SteemConnectAccount

    def get_default_scope(self):
        return ['vote', 'comment', 'comment_options', 'custom_json']

    def extract_uid(self, data):
        return str(data['account']['id'])


provider_classes = [SteemConnectProvider]

providers.registry.register(SteemConnectProvider)



import json

import requests
from allauth.socialaccount.providers.oauth2.views import (
    OAuth2Adapter,
    OAuth2CallbackView,
    OAuth2LoginView,
)

from django-allauth-steemconnect.providers.steemconnect.provider import SteemConnectProvider


class SteemConnectOAuth2Adapter(OAuth2Adapter):
    provider_id = SteemConnectProvider.id
    access_token_url = 'https://steemconnect.com/api/oauth2/token'
    authorize_url = 'https://steemconnect.com/oauth2/authorize'
    revoke_url = 'https://steemconnect.com/api/oauth2/token/revoke'
    profile_url = 'https://steemconnect.com/api/me'
    scope_delimiter = ','

    def complete_login(self, request, app, token, **kwargs):
        resp = requests.get(self.profile_url, headers={
            'Authorization': token.token
        })
        extra_data = json.loads(resp.content)
        return self.get_provider().sociallogin_from_response(
            request,
            extra_data
        )


oauth2_login = OAuth2LoginView.adapter_view(SteemConnectOAuth2Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(SteemConnectOAuth2Adapter)



from allauth.socialaccount.tests import OAuth2TestsMixin
from allauth.tests import MockedResponse, TestCase

from .provider import SteemConnectProvider


class SteemConnectTests(OAuth2TestsMixin, TestCase):
    provider_id = SteemConnectProvider.id

    def get_mocked_response(self):
        return MockedResponse(200, """
        {
          "user": "ohing504",
          "_id": "ohing504",
          "name": "ohing504",
          "account": {
            "id": 1083713,
            "name": "ohing504",
            "owner": {
              "weight_threshold": 1,
              "account_auths": [],
              "key_auths": [
                [
                  "DSfajioewfnweAFfdsaijofjwel21rjfojoaFAewjfoiewjf12fwo",
                  1
                ]
              ]
            },
            "active": {
              "weight_threshold": 1,
              "account_auths": [],
              "key_auths": [
                [
                  "DSfajioewfnweAFfdsaijofjwel21rjfojoaFAewjfoiewjf12fwo",
                  1
                ]
              ]
            },
            "posting": {
              "weight_threshold": 1,
              "account_auths": [
                [
                  "steemhunt.com",
                  1
                ],
                [
                  "tasteem.app",
                  1
                ]
              ],
              "key_auths": [
                [
                  "DSfajioewfnweAFfdsaijofjwel21rjfojoaFAewjfoiewjf12fwo",
                  1
                ]
              ]
            },
            "memo_key": "DSfajioewfnweAFfdsaijofjwel21rjfojoaFAewjfoiewjf12fwo",
            "json_metadata": "{\"profile\":{\"profile_image\":\"https://cdn.steemitimages.com/DSfajioewfnweAFfdsaijofjwel21rjfojoaFAewjfoiewjf12fwo/KakaoTalk.jpeg\"}}",
            "proxy": "",
            "last_owner_update": "1970-01-01T00:00:00",
            "last_account_update": "2018-11-08T01:14:33",
            "created": "2018-07-27T12:36:42",
            "mined": false,
            "recovery_account": "steem",
            "last_account_recovery": "1970-01-01T00:00:00",
            "reset_account": "null",
            "comment_count": 0,
            "lifetime_vote_count": 0,
            "post_count": 3,
            "can_vote": true,
            "voting_manabar": {
              "current_mana": "29602434198",
              "last_update_time": 1543762779
            },
            "voting_power": 9799,
            "balance": "0.000 STEEM",
            "savings_balance": "0.000 STEEM",
            "sbd_balance": "0.001 SBD",
            "sbd_seconds": "0",
            "sbd_seconds_last_update": "2018-11-22T07:03:36",
            "sbd_last_interest_payment": "1970-01-01T00:00:00",
            "savings_sbd_balance": "0.000 SBD",
            "savings_sbd_seconds": "0",
            "savings_sbd_seconds_last_update": "1970-01-01T00:00:00",
            "savings_sbd_last_interest_payment": "1970-01-01T00:00:00",
            "savings_withdraw_requests": 0,
            "reward_sbd_balance": "0.012 SBD",
            "reward_steem_balance": "3.377 STEEM",
            "reward_vesting_balance": "6869.641284 VESTS",
            "reward_vesting_steem": "3.411 STEEM",
            "vesting_shares": "202.721756 VESTS",
            "delegated_vesting_shares": "0.000000 VESTS",
            "received_vesting_shares": "30003.843753 VESTS",
            "vesting_withdraw_rate": "0.000000 VESTS",
            "next_vesting_withdrawal": "1969-12-31T23:59:59",
            "withdrawn": 0,
            "to_withdraw": 0,
            "withdraw_routes": 0,
            "curation_rewards": 0,
            "posting_rewards": 6821,
            "proxied_vsf_votes": [
              0,
              0,
              0,
              0
            ],
            "witnesses_voted_for": 0,
            "last_post": "2018-11-22T06:59:30",
            "last_root_post": "2018-11-22T06:59:30",
            "last_vote_time": "2018-12-02T14:59:39",
            "post_bandwidth": 0,
            "pending_claimed_accounts": 0,
            "vesting_balance": "0.000 STEEM",
            "reputation": "102070271724",
            "transfer_history": [],
            "market_history": [],
            "post_history": [],
            "vote_history": [],
            "other_history": [],
            "witness_votes": [],
            "tags_usage": [],
            "guest_bloggers": []
          },
          "scope": [
            "login",
            "vote",
            "comment",
            "delete_comment",
            "comment_options",
            "custom_json"
          ],
          "user_metadata": {}
        }
        """)
